@@ -35,7 +35,7 @@ Listeners should take the following arguments:
 * *method*: HTTP Request method used
 * *path*: Path to the URL that was called (or override name if it was used in the call to the client)
 * *response_time*: Reponse time in milliseconds
-* *response*: Instance of locust.clients.HttpResponse
+* *response_length*: Content-length of the response
 """
 
 request_failure = EventHook()
@@ -60,6 +60,7 @@ Event is fired with the following arguments:
 
 * *locust_instance*: Locust class instance where the exception occured
 * *exception*: Exception that was thrown
+* *traceback*: Traceback object (from sys.exc_info()[2])
 """
 
 report_to_master = EventHook()
