@@ -1,10 +1,9 @@
 import time
 import gevent
 from copy import copy
-import math
 
 import events
-from exception import InterruptLocust
+from exception import StopLocust
 from log import console_logger
 
 STATS_NAME_WIDTH = 60
@@ -50,7 +49,7 @@ class RequestStats(object):
         self.num_failures = 0
         self.total_response_time = 0
         self.response_times = {}
-        self._min_response_time = None
+        self.min_response_time = 0
         self.max_response_time = 0
         self.last_request_timestamp = int(time.time())
         self.num_reqs_per_sec = {}
@@ -58,19 +57,27 @@ class RequestStats(object):
 
     def log(self, response_time, content_length):
         RequestStats.total_num_requests += 1
-
         self.num_reqs += 1
-        self.total_response_time += response_time
 
+        self.log_request_time()
+        self.log_response_time(response_time)
+
+        # increase total content-length
+        self.total_content_length += content_length
+
+    def log_request_time(self):
         t = int(time.time())
         self.num_reqs_per_sec[t] = self.num_reqs_per_sec.setdefault(t, 0) + 1
         self.last_request_timestamp = t
         RequestStats.global_last_request_timestamp = t
 
-        if self._min_response_time is None:
-            self._min_response_time = response_time
+    def log_response_time(self, response_time):
+        self.total_response_time += response_time
 
-        self._min_response_time = min(self._min_response_time, response_time)
+        if self.min_response_time is None:
+            self.min_response_time = response_time
+
+        self.min_response_time = min(self.min_response_time, response_time)
         self.max_response_time = max(self.max_response_time, response_time)
 
         # to avoid to much data that has to be transfered to the master node when
@@ -84,13 +91,11 @@ class RequestStats(object):
             rounded_response_time = int(round(response_time, -2))
         else:
             rounded_response_time = int(round(response_time, -3))
+
         # increase request count for the rounded key in response time dict
         self.response_times.setdefault(rounded_response_time, 0)
         self.response_times[rounded_response_time] += 1
 
-        # increase total content-length
-        self.total_content_length += content_length
-
     def log_error(self, error):
         self.num_failures += 1
         key = "%r: %s" % (error, repr(str(error)))
@@ -107,12 +112,6 @@ class RequestStats(object):
             else:
                 return 0.0
 
-    @property
-    def min_response_time(self):
-        if self._min_response_time is None:
-            return 0
-        return self._min_response_time
-
     @property
     def avg_response_time(self):
         try:
@@ -124,9 +123,9 @@ class RequestStats(object):
     def median_response_time(self):
         if not self.response_times:
             return 0
-        
+
         return median_from_dict(self.num_reqs, self.response_times)
-    
+
     @property
     def current_rps(self):
         if self.global_last_request_timestamp is None:
@@ -149,19 +148,20 @@ class RequestStats(object):
             return self.total_content_length / self.num_reqs
         except ZeroDivisionError:
             return 0
-        
+
     def __iadd__(self, other):
         self.iadd_stats(other)
-    
+        return self
+
     def iadd_stats(self, other, full_request_history=False):
         self.last_request_timestamp = max(self.last_request_timestamp, other.last_request_timestamp)
         self.start_time = min(self.start_time, other.start_time)
-        
+
         self.num_reqs = self.num_reqs + other.num_reqs
         self.num_failures = self.num_failures + other.num_failures
         self.total_response_time = self.total_response_time + other.total_response_time
         self.max_response_time = max(self.max_response_time, other.max_response_time)
-        self._min_response_time = min(self._min_response_time, other._min_response_time) or other._min_response_time
+        self.min_response_time = min(self.min_response_time, other.min_response_time) or other.min_response_time
         self.total_content_length = self.total_content_length + other.total_content_length
 
         if full_request_history:
@@ -207,28 +207,33 @@ class RequestStats(object):
             self.current_rps or 0
         )
 
-    def create_response_times_list(self):
-        inflated_list = []
-        for response_time, count in self.response_times.iteritems():
-            inflated_list.extend([response_time for x in xrange(0, count)])
-        inflated_list.sort()
+    def get_response_time_percentile(self, percent):
+        """
+        Percent specified in range: 0.0 - 1.0
+        """
+        num_of_request = int((self.num_reqs * percent))
 
-        return inflated_list
+        processed_count = 0
+        for response_time in sorted(self.response_times.iterkeys(), reverse=True):
+            processed_count += self.response_times[response_time]
+            if((self.num_reqs - processed_count) <= num_of_request):
+                return response_time
 
     def percentile(self, tpl=" %-" + str(STATS_NAME_WIDTH) + "s %8d %6d %6d %6d %6d %6d %6d %6d %6d %6d"):
-        inflated_list = self.create_response_times_list()
+        if not self.num_reqs:
+            raise ValueError("Can't calculate percentile on url with no successful requests")
         return tpl % (
             self.name,
             self.num_reqs,
-            percentile(inflated_list, 0.5),
-            percentile(inflated_list, 0.66),
-            percentile(inflated_list, 0.75),
-            percentile(inflated_list, 0.80),
-            percentile(inflated_list, 0.90),
-            percentile(inflated_list, 0.95),
-            percentile(inflated_list, 0.98),
-            percentile(inflated_list, 0.99),
-            percentile(inflated_list, 1.0)
+            self.get_response_time_percentile(0.5),
+            self.get_response_time_percentile(0.66),
+            self.get_response_time_percentile(0.75),
+            self.get_response_time_percentile(0.80),
+            self.get_response_time_percentile(0.90),
+            self.get_response_time_percentile(0.95),
+            self.get_response_time_percentile(0.98),
+            self.get_response_time_percentile(0.99),
+            self.max_response_time
         )
 
     @classmethod
@@ -260,34 +265,11 @@ def median_from_dict(total, count):
             return k
         pos -= count[k]
 
-# TODO Use interpolation or not?
-def percentile(N, percent, key=lambda x:x):
-    """
-    Find the percentile of a list of values.
-
-    @parameter N - is a list of values. Note N MUST BE already sorted.
-    @parameter percent - a float value from 0.0 to 1.0.
-    @parameter key - optional key function to compute value from each element of N.
-
-    @return - the percentile of the values
-    """
-    if not N:
-        return 0
-    k = (len(N)-1) * percent
-    f = math.floor(k)
-    c = math.ceil(k)
-    if f == c:
-        return key(N[int(k)])
-    d0 = key(N[int(f)]) * (c-k)
-    d1 = key(N[int(c)]) * (k-f)
-    return d0+d1
-
-def on_request_success(method, name, response_time, response):
+def on_request_success(method, name, response_time, response_length):
     if RequestStats.global_max_requests is not None and RequestStats.total_num_requests >= RequestStats.global_max_requests:
-        raise InterruptLocust("Maximum number of requests reached")
-
-    content_length = int(response.info.getheader("Content-Length") or 0)
-    RequestStats.get(method, name).log(response_time, content_length)
+        raise StopLocust("Maximum number of requests reached")
+    
+    RequestStats.get(method, name).log(response_time, response_length)
 
 def on_request_failure(method, name, response_time, error, response=None):
     RequestStats.get(method, name).log_error(error)
@@ -340,28 +322,15 @@ def print_percentile_stats(stats):
     console_logger.info("Percentage of the requests completed within given times")
     console_logger.info((" %-" + str(STATS_NAME_WIDTH) + "s %8s %6s %6s %6s %6s %6s %6s %6s %6s %6s") % ('Name', '# reqs', '50%', '66%', '75%', '80%', '90%', '95%', '98%', '99%', '100%'))
     console_logger.info("-" * (80 + STATS_NAME_WIDTH))
-    complete_list = []
+    total_stats = RequestStats("", "Total")
     for key in sorted(stats.iterkeys()):
         r = stats[key]
         if r.response_times:
             console_logger.info(r.percentile())
-            complete_list.extend(r.create_response_times_list())
+            total_stats += r
     console_logger.info("-" * (80 + STATS_NAME_WIDTH))
-    complete_list.sort()
-    if complete_list:
-        console_logger.info( (" %-" + str(STATS_NAME_WIDTH) + "s %8s %6d %6d %6d %6d %6d %6d %6d %6d %6d") % (
-            'Total',
-            str(len(complete_list)),
-            percentile(complete_list, 0.5),
-            percentile(complete_list, 0.66),
-            percentile(complete_list, 0.75),
-            percentile(complete_list, 0.8),
-            percentile(complete_list, 0.9),
-            percentile(complete_list, 0.95),
-            percentile(complete_list, 0.98),
-            percentile(complete_list, 0.99),
-            complete_list[-1]
-        ))
+    if total_stats.response_times:
+        console_logger.info(total_stats.percentile())
     console_logger.info("")
 
 def print_error_report():
