@@ -1,9 +1,11 @@
 import unittest
 import time
 
+from requests.exceptions import RequestException
+
 from testcases import WebserverTestCase
 from locust.stats import RequestStats
-from locust.core import Locust, SubLocust, task
+from locust.core import Locust, TaskSet, task
 from locust.inspectlocust import get_task_ratio_dict
 
 class TestRequestStats(unittest.TestCase):
@@ -21,6 +23,15 @@ class TestRequestStats(unittest.TestCase):
         self.s.log(79, 0)
         self.s.log_error(Exception("dummy fail"))
 
+    def test_percentile(self):
+        s = RequestStats("GET", "percentile_test")
+        for x in xrange(100):
+            s.log(x, 0)
+
+        self.assertEqual(s.get_response_time_percentile(0.5), 50)
+        self.assertEqual(s.get_response_time_percentile(0.6), 60)
+        self.assertEqual(s.get_response_time_percentile(0.95), 95)
+
     def test_median(self):
         self.assertEqual(self.s.median_response_time, 79)
 
@@ -90,15 +101,57 @@ class TestRequestStatsWithWebserver(WebserverTestCase):
         self.assertEqual(RequestStats.get("GET", "/ultra_fast").avg_content_length, len("This is an ultra fast response"))
         locust.client.get("/ultra_fast")
         self.assertEqual(RequestStats.get("GET", "/ultra_fast").avg_content_length, len("This is an ultra fast response"))
+    
+    def test_request_stats_no_content_length(self):
+        class MyLocust(Locust):
+            host = "http://127.0.0.1:%i" % self.port
+        l = MyLocust()
+        path = "/no_content_length"
+        r = l.client.get(path)
+        self.assertEqual(RequestStats.get("GET", path).avg_content_length, len("This response does not have content-length in the header"))
+    
+    def test_request_stats_no_content_length_no_prefetch(self):
+        class MyLocust(Locust):
+            host = "http://127.0.0.1:%i" % self.port
+        l = MyLocust()
+        path = "/no_content_length"
+        r = l.client.get(path, prefetch=False)
+        self.assertEqual(0, RequestStats.get("GET", path).avg_content_length)
+    
+    def test_request_stats_named_endpoint(self):
+        class MyLocust(Locust):
+            host = "http://127.0.0.1:%i" % self.port
+    
+        locust = MyLocust()
+        locust.client.get("/ultra_fast", name="my_custom_name")
+        self.assertEqual(1, RequestStats.get("GET", "my_custom_name").num_reqs)
+    
+    def test_request_stats_query_variables(self):
+        class MyLocust(Locust):
+            host = "http://127.0.0.1:%i" % self.port
+    
+        locust = MyLocust()
+        locust.client.get("/ultra_fast?query=1")
+        self.assertEqual(1, RequestStats.get("GET", "/ultra_fast?query=1").num_reqs)
+    
+    def test_request_connection_error(self):
+        class MyLocust(Locust):
+            host = "http://localhost:1"
+        
+        locust = MyLocust()
+        response = locust.client.get("/", timeout=0.1)
+        self.assertFalse(response)
+        self.assertEqual(1, RequestStats.get("GET", "/").num_failures)
+        self.assertEqual(0, RequestStats.get("GET", "/").num_reqs)
 
 
-class MyLocust(Locust):
+class MyTaskSet(TaskSet):
     @task(75)
     def root_task(self):
         pass
     
     @task(25)
-    class MySubLocust(SubLocust):
+    class MySubTaskSet(TaskSet):
         @task
         def task1(self):
             pass
@@ -108,18 +161,18 @@ class MyLocust(Locust):
     
 class TestInspectLocust(unittest.TestCase):
     def test_get_task_ratio_dict_relative(self):
-        ratio = get_task_ratio_dict([MyLocust])
-        self.assertEqual(1.0, ratio["MyLocust"]["ratio"])
-        self.assertEqual(0.75, ratio["MyLocust"]["tasks"]["root_task"]["ratio"])
-        self.assertEqual(0.25, ratio["MyLocust"]["tasks"]["MySubLocust"]["ratio"])
-        self.assertEqual(0.5, ratio["MyLocust"]["tasks"]["MySubLocust"]["tasks"]["task1"]["ratio"])
-        self.assertEqual(0.5, ratio["MyLocust"]["tasks"]["MySubLocust"]["tasks"]["task2"]["ratio"])
+        ratio = get_task_ratio_dict([MyTaskSet])
+        self.assertEqual(1.0, ratio["MyTaskSet"]["ratio"])
+        self.assertEqual(0.75, ratio["MyTaskSet"]["tasks"]["root_task"]["ratio"])
+        self.assertEqual(0.25, ratio["MyTaskSet"]["tasks"]["MySubTaskSet"]["ratio"])
+        self.assertEqual(0.5, ratio["MyTaskSet"]["tasks"]["MySubTaskSet"]["tasks"]["task1"]["ratio"])
+        self.assertEqual(0.5, ratio["MyTaskSet"]["tasks"]["MySubTaskSet"]["tasks"]["task2"]["ratio"])
     
     def test_get_task_ratio_dict_total(self):
-        ratio = get_task_ratio_dict([MyLocust], total=True)
-        self.assertEqual(1.0, ratio["MyLocust"]["ratio"])
-        self.assertEqual(0.75, ratio["MyLocust"]["tasks"]["root_task"]["ratio"])
-        self.assertEqual(0.25, ratio["MyLocust"]["tasks"]["MySubLocust"]["ratio"])
-        self.assertEqual(0.125, ratio["MyLocust"]["tasks"]["MySubLocust"]["tasks"]["task1"]["ratio"])
-        self.assertEqual(0.125, ratio["MyLocust"]["tasks"]["MySubLocust"]["tasks"]["task2"]["ratio"])
+        ratio = get_task_ratio_dict([MyTaskSet], total=True)
+        self.assertEqual(1.0, ratio["MyTaskSet"]["ratio"])
+        self.assertEqual(0.75, ratio["MyTaskSet"]["tasks"]["root_task"]["ratio"])
+        self.assertEqual(0.25, ratio["MyTaskSet"]["tasks"]["MySubTaskSet"]["ratio"])
+        self.assertEqual(0.125, ratio["MyTaskSet"]["tasks"]["MySubTaskSet"]["tasks"]["task1"]["ratio"])
+        self.assertEqual(0.125, ratio["MyTaskSet"]["tasks"]["MySubTaskSet"]["tasks"]["task2"]["ratio"])
         
\ No newline at end of file
