@@ -1,10 +1,14 @@
+import random
 import gevent
 import gevent.pywsgi
 import unittest
-from werkzeug.wrappers import Request
 import base64
+from copy import copy
+from StringIO import StringIO
+
+from locust import events
 from locust.stats import RequestStats
-from flask import Flask, request, redirect, make_response
+from flask import Flask, request, redirect, make_response, send_file
 
 app = Flask(__name__)
 
@@ -57,12 +61,39 @@ def basic_auth():
     resp.headers["WWW-Authenticate"] = 'Basic realm="Locust"'
     return resp
 
+@app.route("/no_content_length")
+def no_content_length():
+    r = send_file(StringIO("This response does not have content-length in the header"))
+    return r
+
 @app.errorhandler(404)
 def not_found(error):
     return "Not Found", 404
 
-class WebserverTestCase(unittest.TestCase):
+
+class LocustTestCase(unittest.TestCase):
+    """
+    Test case class that restores locust.events.EventHook listeners on tearDown, so that it is
+    safe to register any custom event handlers within the test.
+    """
     def setUp(self):
+        self._event_handlers = {}
+        for name in dir(events):
+            event = getattr(events, name)
+            if isinstance(event, events.EventHook):
+                self._event_handlers[event] = copy(event._handlers)
+                      
+    def tearDown(self):
+        for event, handlers in self._event_handlers.iteritems():
+            event._handlers = handlers
+
+            
+class WebserverTestCase(LocustTestCase):
+    """
+    Test case class that sets up an HTTP server which can be used within the tests
+    """
+    def setUp(self):
+        super(WebserverTestCase, self).setUp()
         self._web_server = gevent.pywsgi.WSGIServer(("127.0.0.1", 0), app, log=None)
         gevent.spawn(lambda: self._web_server.serve_forever())
         gevent.sleep(0)
@@ -70,4 +101,5 @@ class WebserverTestCase(unittest.TestCase):
         RequestStats.requests = {}
 
     def tearDown(self):
+        super(WebserverTestCase, self).tearDown()
         self._web_server.kill()
